"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var ts_express_decorators_1 = require("ts-express-decorators");
var ts_log_debug_1 = require("ts-log-debug");
var UsersService_1 = require("../../services/UsersService");
var ts_httpexceptions_1 = require("ts-httpexceptions");
var UserCtrl = (function () {
    function UserCtrl(usersService) {
        this.usersService = usersService;
    }
    /**
     * Authenticate a user.
     * @param email
     * @param password
     * @returns {IUser}
     */
    UserCtrl.prototype.authenticate = function (email, password) {
        ts_log_debug_1.$log.debug("authenticate user with email", email, " & password ", password);
        var user = this.usersService.findByEmail(email);
        ts_log_debug_1.$log.debug("find user by email", user);
        if (!user) {
            throw new ts_httpexceptions_1.NotFound("authentication failed, user not found");
        }
        if (user.password !== password) {
            throw new ts_httpexceptions_1.Unauthorized("authentication failed, wrong password");
        }
        return this.usersService.update(user._id, { status: "online" });
    };
    /**
     * Find a user by is mail or id.
     * @param idOrMail
     * @returns {IUser}
     */
    UserCtrl.prototype.get = function (idOrMail) {
        var user = this.usersService.findByEmail(idOrMail) || this.usersService.find(idOrMail);
        if (!user) {
            throw new ts_httpexceptions_1.NotFound("User not found.");
        }
        return user;
    };
    UserCtrl.prototype.updateStatus = function (email, status) {
        if (!this.usersService.checkStatus(status)) {
            throw new ts_httpexceptions_1.BadRequest("Wrong status");
        }
        if (!this.usersService.checkEmail(email)) {
            throw new ts_httpexceptions_1.BadRequest("Wrong email");
        }
        var user = this.usersService.findByEmail(email);
        if (!user) {
            throw new ts_httpexceptions_1.NotFound("User not found.");
        }
        ts_log_debug_1.$log.debug("patch from email", email, "with status", status);
        return this.usersService.update(user._id, { status: status });
    };
    /**
     *
     * @param id
     * @param user
     * @returns {IUser}
     */
    UserCtrl.prototype.update = function (id, user) {
        var oldUser = this.usersService.find(id);
        if (!oldUser) {
            throw new ts_httpexceptions_1.NotFound("User not found.");
        }
        return this.usersService.update(oldUser._id, user);
    };
    /**
     *
     * @param user
     * @returns {IUser}
     */
    UserCtrl.prototype.create = function (user) {
        ts_log_debug_1.$log.debug("rest create user", user);
        if (!user.email || !this.usersService.checkEmail(user.email)) {
            throw new ts_httpexceptions_1.BadRequest("Email are required");
        }
        if (!user.password) {
            throw new ts_httpexceptions_1.BadRequest("Password are required");
        }
        if (this.usersService.findByEmail(user.email)) {
            throw new ts_httpexceptions_1.BadRequest("User already created with this email.");
        }
        return this.usersService.create(user);
    };
    /**
     * Remove the user.
     * @param id
     * @returns {any}
     */
    UserCtrl.prototype.remove = function (id) {
        var user = this.usersService.find(id);
        if (!user) {
            throw new ts_httpexceptions_1.NotFound("User not found.");
        }
        this.usersService.remove(id);
        return user;
    };
    /**
     * Get All users.
     * @returns {IUser[]}
     */
    UserCtrl.prototype.getList = function () {
        return this.usersService.query().map(function (o) {
            o = Object.assign({}, o);
            delete o.password;
            return o;
        });
    };
    return UserCtrl;
}());
__decorate([
    ts_express_decorators_1.Post("/authenticate"),
    __param(0, ts_express_decorators_1.Required()), __param(0, ts_express_decorators_1.BodyParams("email")),
    __param(1, ts_express_decorators_1.Required()), __param(1, ts_express_decorators_1.BodyParams("password")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", void 0)
], UserCtrl.prototype, "authenticate", null);
__decorate([
    ts_express_decorators_1.Get("/:id"),
    __param(0, ts_express_decorators_1.PathParams("id")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Object)
], UserCtrl.prototype, "get", null);
__decorate([
    ts_express_decorators_1.Patch("/:email/:status"),
    __param(0, ts_express_decorators_1.Required()), __param(0, ts_express_decorators_1.PathParams("email")),
    __param(1, ts_express_decorators_1.Required()), __param(1, ts_express_decorators_1.PathParams("status")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Object)
], UserCtrl.prototype, "updateStatus", null);
__decorate([
    ts_express_decorators_1.Put("/:id"),
    __param(0, ts_express_decorators_1.PathParams("id")),
    __param(1, ts_express_decorators_1.Required()), __param(1, ts_express_decorators_1.BodyParams("user")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", Object)
], UserCtrl.prototype, "update", null);
__decorate([
    ts_express_decorators_1.Post("/"),
    ts_express_decorators_1.Status(201),
    __param(0, ts_express_decorators_1.Required()), __param(0, ts_express_decorators_1.BodyParams("user")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Object)
], UserCtrl.prototype, "create", null);
__decorate([
    ts_express_decorators_1.Delete("/:id"),
    __param(0, ts_express_decorators_1.PathParams("id")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", void 0)
], UserCtrl.prototype, "remove", null);
__decorate([
    ts_express_decorators_1.Get("/"),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Array)
], UserCtrl.prototype, "getList", null);
UserCtrl = __decorate([
    ts_express_decorators_1.Controller("/users"),
    __metadata("design:paramtypes", [UsersService_1.UsersService])
], UserCtrl);
exports.UserCtrl = UserCtrl;
//# sourceMappingURL=UserCtrl.js.map