"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var ts_json_properties_1 = require("ts-json-properties");
var ts_express_decorators_1 = require("ts-express-decorators");
var ts_log_debug_1 = require("ts-log-debug");
var SocketService_1 = require("./SocketService");
var UsersService = (function () {
    function UsersService(socketService) {
        this.socketService = socketService;
        this.users = this.USERS.map(function (u) { return Object.assign({}, u); });
    }
    /**
     *
     * @param email
     * @returns {boolean}
     */
    UsersService.prototype.checkEmail = function (email) {
        return !!email.match(/^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/);
    };
    /**
     *
     * @param status
     * @returns {boolean}
     */
    UsersService.prototype.checkStatus = function (status) {
        return ["offline", "busy", "online"].indexOf(status) !== -1;
    };
    /**
     * Find a user by his ID.
     * @param id
     * @returns {undefined|IUser}
     */
    UsersService.prototype.find = function (id) {
        var users = this.query();
        return users.find(function (value) { return value._id === id; });
    };
    UsersService.prototype.findByEmail = function (email) {
        ts_log_debug_1.$log.debug("find by email", email);
        var users = this.query();
        ts_log_debug_1.$log.debug("users size", users.length);
        var user = users.find(function (user) { return user.email === email; });
        ts_log_debug_1.$log.debug("user found", user);
        return user;
    };
    UsersService.prototype.findByCredential = function (email, password) {
        var users = this.query();
        return users.find(function (value) { return value.email === email && value.password === password; });
    };
    /**
     * Create a new User
     * @returns {{id: any, name: string}}
     * @param user
     */
    UsersService.prototype.create = function (user) {
        user._id = require("node-uuid").v4();
        ts_log_debug_1.$log.debug("create user", user);
        ts_log_debug_1.$log.debug("users size before insert", this.users.length);
        this.users.push(user);
        ts_log_debug_1.$log.debug("users size", this.users.length);
        this.socketService.emit("users.update", this.query());
        return user;
    };
    /**
     *
     * @returns {IUser[]}
     */
    UsersService.prototype.query = function () {
        return this.users;
    };
    /**
     *
     * @param id
     * @param user
     * @returns {IUser}
     */
    UsersService.prototype.update = function (id, user) {
        var users = this.query();
        ts_log_debug_1.$log.debug("users size", users.length);
        var index = this.users.findIndex(function (o) { return id === o._id; });
        ts_log_debug_1.$log.debug("users index", index);
        users[index] = Object.assign(users[index], user);
        this.socketService.emit("users.update", this.query());
        return users[index];
    };
    UsersService.prototype.remove = function (id) {
        var index = this.users.findIndex(function (o) { return id === o._id; });
        this.users.splice(index, 1);
        this.socketService.emit("users.update", this.query());
    };
    return UsersService;
}());
__decorate([
    ts_json_properties_1.Value("users.list"),
    __metadata("design:type", Array)
], UsersService.prototype, "USERS", void 0);
UsersService = __decorate([
    ts_express_decorators_1.Service(),
    __metadata("design:paramtypes", [SocketService_1.SocketService])
], UsersService);
exports.UsersService = UsersService;
//# sourceMappingURL=UsersService.js.map